/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.defne.service;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.defne.utility.Assertions;

/**
 * Each service that is located in the server side, is called using the
 * {@link Message} that will be marshalled to XML-RPC calls.
 * <p>
 * Each {@link Message} contains the
 * <ul>
 * <li>Name of the service that is located in the service</li>
 * <li>And parameters of the service</li>
 * </ul>
 * </p>
 * 
 * @author gurkanerdogdu
 * @version $Rev$ $Date$
 */
public class Message implements Serializable,Cloneable
{
    // Serial ID as default
    private static final long serialVersionUID = 1L;

    /** Service bag parameters */
    private Map<String, MessageParam<?>> bagParameters = new HashMap<String, MessageParam<?>>();
    
    /**Returns outBag or not*/
    private boolean returnsOutMessage = true;
    
    /**
     * Creates an empty service bag.
     */
    Message()
    {
        
    }
    
    /**
     * Creates a new service bag with given service name.
     * 
     * @param serviceName service name of the bag
     */
    Message(String serviceName)
    {
        bagParameters.put(IMessageConstants.SERVICE_NAME_KEY, MessageParamFactory.newMessageParam(serviceName));
    }

    /**
     * Creates a new service bag with given service name and method name.
     * 
     * @param serviceName service name of the bag
     * @param methodName method name of the service
     */
    Message(String serviceName, String methodName)
    {
        bagParameters.put(IMessageConstants.SERVICE_NAME_KEY, MessageParamFactory.newMessageParam(serviceName));
        bagParameters.put(IMessageConstants.SERVICE_METHOD_NAME_KEY, MessageParamFactory.newMessageParam(methodName));
    }

    /**
     * Puts a new bag parameter.
     * 
     * @param <T> bag parameter payload generic type
     * @param messageParameterKey bag parameter key
     * @param messageParameter bag parameter payload
     */
    public <T> void putMessageParameter(String messageParameterKey, T messageParameter)
    {
        Assertions.checkForNull(messageParameterKey, "messageParameterKey is null");
        Assertions.checkForNull(messageParameter, "messageParameter is null");

        // Create value
        MessageParam<T> bagValue = new MessageParam<T>(messageParameter);

        bagParameters.put(messageParameterKey, bagValue);
    }
    
    /**
     * Puts a new bag parameter.
     * 
     * @param <T> bag parameter generic type
     * @param bagParameterKey bag parameter key
     * @param bagParameterValue bag parameter
     */
    public <T> void putMessageParameter(String bagParameterKey, MessageParam<T> bagParameterValue)
    {
        Assertions.checkForNull(bagParameterKey, "bagParameterKey is null");
        Assertions.checkForNull(bagParameterValue, "bagParameterValue is null");

        bagParameters.put(bagParameterKey, bagParameterValue);
    }

    /**
     * Removes the given bag parameter.
     * 
     * @param bagParameterKey bag parameter key
     */
    public void removeMessageParameter(String bagParameterKey)
    {
        if (bagParameterKey != null)
        {
            bagParameters.remove(bagParameterKey);
        }
    }

    /**
     * Gets the service name of this service bag.
     * 
     * @return the service name of this service bag
     */
    public String getServiceName()
    {
        return getMessageParameter(String.class, IMessageConstants.SERVICE_NAME_KEY);
    }

    /**
     * Gets the service method name of this service bag.
     * 
     * @return the service method name of this service bag
     */
    public String getServiceMethodName()
    {
        return getMessageParameter(String.class, IMessageConstants.SERVICE_METHOD_NAME_KEY);
    }

    /**
     * Gets the message parameter with the given key.
     * @param <T> bag payload class type
     * @param clazz bag payload class
     * @param bagKey key of the bag value
     * @return the payload of the bag value
     */
    @SuppressWarnings("unchecked")
    public <T> T getMessageParameter(Class<T> clazz, String messageKey)
    {
        Assertions.checkForNull(messageKey, "messageKey is null");
        Assertions.checkForNull(clazz, "clazz is null");

        if (bagParameters.containsKey(messageKey))
        {
            MessageParam<T> v = (MessageParam<T>) bagParameters.get(messageKey);
            return v!= null ? v.getValue() : null;
        }

        return null;
    }
    
    /**
     * 
     * @return the list of error messages
     */
    @SuppressWarnings("unchecked")
    public List<Object> getErrorMessages()
    {
        return getMessageParameter(List.class, IMessageConstants.SERVICE_EXECUTION_ERROR_MESSAGES);
    }

    /**
     * 
     * @return the list of warning messages
     */
    @SuppressWarnings("unchecked")
    public List<Object> getWarningMessages()
    {
        return getMessageParameter(List.class, IMessageConstants.SERVICE_EXECUTION_WARNING_MESSAGES);
    }

    /**
     * 
     * @return the list of info messages
     */
    @SuppressWarnings("unchecked")
    public List<Object> getInfoMessages()
    {
        return getMessageParameter(List.class, IMessageConstants.SERVICE_EXECUTION_INFO_MESSAGES);
    }

    /**
     * Gets cause of the exception that may occured
     * in calling service.
     * @return cause of the exception
     */
    public Throwable getException()
    {
        return getMessageParameter(Throwable.class, IMessageConstants.SERVICE_EXECUTION_EXCEPTION_KEY);
    }
    
    /**
     * Gets exception message.
     * @return exception message
     */
    public String getExceptionMessage()
    {
        return getMessageParameter(String.class, IMessageConstants.SERVICE_EXECUTION_EXCEPTION_MESSAGE_KEY);
    }
    
    /**
     * Returns true if the given key exists in the message.
     * @param key message key
     * @return true if given key exist in the message
     */
    public boolean isExist(String key)
    {
        Assertions.checkForNull(key, "key is null");
        return this.bagParameters.containsKey(key);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object clone() throws CloneNotSupportedException
    {
        return super.clone();
    }

    /**
     * 
     * @return true if set
     */
    public boolean isReturnsOutMessage()
    {
        return returnsOutMessage;
    }

    /**
     * If the calling service is not required to return
     * {@link Message} than this parameter is set as true.
     * @param returnsOutMessage flag
     */
    public void setReturnsOutMessage(boolean returnsOutBag)
    {
        this.returnsOutMessage = returnsOutBag;
    }
    
    /**
     * Sets transaction policy.
     * @param transactionPolicy transaction policy
     */
    public void setTransactionPolicy(TransactionPolicy transactionPolicy)
    {
       putMessageParameter(IMessageConstants.SERVICE_CALLER_TRANSACTION_POLICY, transactionPolicy);
    }
    
    /**
     * {@inheritDoc}
     */
    public String toString()
    {
        StringBuilder builder = new StringBuilder("Service Name [ " + getServiceName() + " ],");
        builder.append("Method Name [ " + getServiceMethodName() + " ]");
        
        return builder.toString();
    }
}
