/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.defne.service.annotation;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;
import java.util.concurrent.CopyOnWriteArrayList;


import org.defne.security.annotation.Roles;
import org.defne.service.TransactionPolicy;
import org.defne.service.scanner.ScannerFactory;
import org.defne.utility.Assertions;
import org.defne.utility.factories.SingletonProvider;
import org.defne.utility.log.ILogger;
import org.defne.utility.log.LoggerProvider;

/**
 * Scanning for service classes.   
 * @version $Rev$ $Date$
 *
 */
public class ServiceScanner
{
    private static final ILogger logger = LoggerProvider.getLogProvider(ServiceScanner.class);
    
    /**Service infos for each classloader*/
    private final Map<ClassLoader, List<ServiceInfo>> serviceInfoMap = 
        Collections.synchronizedMap(new WeakHashMap<ClassLoader, List<ServiceInfo>>());
    
    /**
     * Default constructor.
     */
    public ServiceScanner()
    {
        //Default
    }
    
    /**
     * Gets singleton service scanner instance.
     * @return the singleton instance
     */
    public static ServiceScanner getInstance()
    {
        return (ServiceScanner)SingletonProvider.getSingletonInstance(SingletonProvider.SERVICE_SCANNER_CLASS);
    }
    
    /**
     * Scans the given classloader for finding services.
     * @param classLoader deployment class loader
     * @throws Exception for any exception
     */
    public void scan(final ClassLoader classLoader) throws Exception
    {

        if (this.serviceInfoMap.containsKey(classLoader))
        {
            return;
        }
        else
        {
            List<ServiceInfo> serviceInfoList = new CopyOnWriteArrayList<ServiceInfo>();
            this.serviceInfoMap.put(classLoader, serviceInfoList);
            
            //Getting scanner factory for deployment
            ScannerFactory scannerFactory = (ScannerFactory)SingletonProvider.getSingletonInstance("org.defne.service.scanner.ScannerFactory", classLoader);            

            Set<Class<?>> annotatedClasses = scannerFactory.getScanner().scan(classLoader);            
            for (Class<?> annotatedClass : annotatedClasses)
            {
                ServiceInfo serviceInfo = new ServiceInfo();
                
                Service greenServiceAnnotation = (Service) annotatedClass.getAnnotation(Service.class);                
                if(!greenServiceAnnotation.name().equals("") && greenServiceAnnotation.name() != null)
                {
                    serviceInfo.serviceName = greenServiceAnnotation.name();    
                }
                else
                {
                    serviceInfo.serviceName = annotatedClass.getSimpleName();
                }
                
                TransactionPolicy classLevel = null;
                if(annotatedClass.isAnnotationPresent(TransactionAttribute.class))
                {
                    classLevel = annotatedClass.getAnnotation(TransactionAttribute.class).value();
                }
                
                EntityManagerAttribute classLevelEmAttr = null;
                if(annotatedClass.isAnnotationPresent(EntityManagerAttribute.class))
                {
                    classLevelEmAttr = annotatedClass.getAnnotation(EntityManagerAttribute.class);
                }
                
                Roles roles = annotatedClass.getAnnotation(Roles.class);
                String[] classRoles = new String[0];
                if(roles != null)
                {
                    classRoles = roles.roles();
                }
                
                Method[] methods = annotatedClass.getDeclaredMethods();
                for (Method method : methods)
                {
                    if(Modifier.isPublic(method.getModifiers()))
                    {
                        if (method.isAnnotationPresent(Operation.class))
                        {
                            //Defining method information
                            ServiceMethod serviceMethod = null;
                            Operation greenMethodAnnotation = method.getAnnotation(Operation.class);
                            if(!greenMethodAnnotation.name().equals("") && greenMethodAnnotation.name() != null)
                            {
                                serviceMethod = new ServiceMethod(greenMethodAnnotation.name(),method);
                                serviceInfo.serviceMethods.add(serviceMethod);    
                            }
                            else
                            {
                                serviceMethod = new ServiceMethod(method.getName(),method);
                                serviceInfo.serviceMethods.add(serviceMethod);    
                            }
                            
                            
                            //Security roles for the method
                            if(method.isAnnotationPresent(Roles.class))
                            {
                                String[] methodRoles = method.getAnnotation(Roles.class).roles();
                                for(String role : classRoles)
                                {
                                    serviceMethod.addRole(role);
                                }
                                
                                for(String role : methodRoles)
                                {
                                    serviceMethod.addRole(role);
                                }
                            }
                            
                            //Class level setting
                            if(classLevel != null)
                            {
                                serviceMethod.setTransactionPolicy(classLevel);
                            }
                            
                            //JPA Entity transactions or not
                            if(method.isAnnotationPresent(TransactionAttribute.class))
                            {
                                TransactionAttribute transAttr = method.getAnnotation(TransactionAttribute.class);
                                serviceMethod.setTransactionPolicy(transAttr.value());
                            }
                            
                            //Class Level
                            if(classLevelEmAttr != null)
                            {
                                serviceMethod.setUseEm(classLevelEmAttr.use());
                                if(classLevelEmAttr.use())
                                {
                                    serviceMethod.setEmUnitName(classLevelEmAttr.unitName());
                                }
                            }
                            
                            //Update class level, with looking for JPA usage or not
                            if(method.isAnnotationPresent(EntityManagerAttribute.class))
                            {
                                EntityManagerAttribute emAttr = method.getAnnotation(EntityManagerAttribute.class);
                                serviceMethod.setUseEm(emAttr.use());
                                if(emAttr.use())
                                {
                                    serviceMethod.setEmUnitName(emAttr.unitName());
                                }
                            }
                        }                        
                    }                    
                }

                serviceInfoList.add(serviceInfo);
            }
        }
    }
    
    /**
     * Gets service method info from given parameters.
     * @param classLoader class loader instance
     * @param serviceName service name
     * @param methodName method name
     * @return service method
     */
    public ServiceMethod getServiceMethod(final ClassLoader classLoader, String serviceName, String methodName)
    {
        Assertions.checkForNull(classLoader, "classloader is null");
        Assertions.checkForNull(serviceName, "servicename is null");
        Assertions.checkForNull(methodName, "methodName is null");
        
        List<ServiceInfo> serviceInfos = this.serviceInfoMap.get(classLoader);        
        if(serviceInfos != null)
        {
            for(ServiceInfo serviceInfo : serviceInfos)
            {
                if(serviceInfo.getServiceName().equals(serviceName))
                {
                     Set<ServiceMethod> serviceMethods = serviceInfo.serviceMethods;
                     for(ServiceMethod serviceMethod : serviceMethods)
                     {
                         if(serviceMethod.methodName.equals(methodName))
                         {
                             return serviceMethod;
                         }
                     }
                }
            }   
        }
        
        return null;
    }
    
    //Service method information
    public static class ServiceMethod
    {
        private String methodName;
        private Method serviceMethod;
        private Set<String> roles = new HashSet<String>();
        private TransactionPolicy transactionPolicy = TransactionPolicy.NO_TRANSACTION;
        private boolean useEm;
        private String emUnitName = "default";
        
        private ServiceMethod(String methodName,Method serviceMethod)
        {
            this.methodName = methodName;
            this.serviceMethod = serviceMethod;
        }
        
        
        public boolean isUseEm()
        {
            return useEm;
        }


        public void setUseEm(boolean useEm)
        {
            this.useEm = useEm;
        }


        public String getEmUnitName()
        {
            return emUnitName;
        }


        public void setEmUnitName(String emUnitName)
        {
            this.emUnitName = emUnitName;
        }

        private void addRole(String role)
        {
            this.roles.add(role);
        }
                

        public String getMethodName()
        {
            return methodName;
        }

        public void setMethodName(String methodName)
        {
            this.methodName = methodName;
        }

        public Method getServiceMethod()
        {
            return serviceMethod;
        }

        public void setServiceMethod(Method serviceMethod)
        {
            this.serviceMethod = serviceMethod;
        }

        public Set<String> getRoles()
        {
            return roles;
        }
        
        public void setTransactionPolicy(TransactionPolicy policy)
        {
            this.transactionPolicy = policy;
        }
        
        public TransactionPolicy getTransactionPolicy()
        {
            return this.transactionPolicy;
        }

        public void setRoles(Set<String> roles)
        {
            this.roles = roles;
        }

        @Override
        public int hashCode()
        {
            final int prime = 31;
            int result = 1;
            result = prime * result + ((methodName == null) ? 0 : methodName.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj)
        {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            ServiceMethod other = (ServiceMethod) obj;
            if (methodName == null)
            {
                if (other.methodName != null)
                    return false;
            }
            else if (!methodName.equals(other.methodName))
                return false;
            return true;
        }
        
        
    }
    
    /**
     * Private class for hodling
     * service informations. 
     */
    public static class ServiceInfo
    {
        private Set<ServiceMethod> serviceMethods = new HashSet<ServiceMethod>();
        private String serviceName;
        
        public String getServiceName()
        {
            return this.serviceName;
        }
        

    }
    
    public void remove(ClassLoader classLoader)
    {
        if(classLoader == null)
        {
            logger.warn("Given classloader is null, nothing to remove from service scanner repository");
            return;
        }
        
        //Remove if exist
        this.serviceInfoMap.remove(classLoader);
        
        //Getting scanner factory for deployment
        ScannerFactory scannerFactory = (ScannerFactory)SingletonProvider.getSingletonInstance("org.defne.service.scanner.ScannerFactory", classLoader);
        
        //Chance to scanner for its own operations
        scannerFactory.getScanner().remove(classLoader);
    }
    
}
