/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.defne.service.executor;

import org.defne.service.Message;
import org.defne.utility.Utility;
import org.defne.utility.exception.DefneException;
import org.defne.utility.log.ILogger;

/**
 * Abstract implementation for the {@link IServiceExecutorProxy} contract.
 * Sub-classes must extend this class instead of directly implementing the
 * {@link IServiceExecutorProxy} interface.
 * @version $Rev$ $Date$
 *
 */
public abstract class AbstractServiceExecutorProxy implements IServiceExecutorProxy
{
    /**Sub-class logger instance*/
    private final ILogger logger; 
    
    /**
     * Used by sub-classes
     * @param logger sub-class logger instance
     */
    protected AbstractServiceExecutorProxy(ILogger logger)
    {
        this.logger = logger;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Message execute(Message inBag) throws DefneException
    {
        Message oBag = null;
        try
        {
            oBag = internalExecute(inBag);
            
        }catch(Exception e)
        {
            Throwable cause = Utility.getTargetException(e);
            if(cause instanceof DefneException)
            {
                throw (DefneException)cause;
            }
            
            throw new DefneException(cause);            
        }
        
        return oBag;
    }

    /**
     * {@inheritDoc}
     */
    public void clear(Object clearInstance)
    {
        try
        {
            internalClear(clearInstance);
            
        }catch(Exception e)
        {
            getLogger().error("Error is occured while clearing the executor", e);
        }
    }
    
    /**
     * Gets logger
     * @return the logger
     */
    protected ILogger getLogger()
    {
        return this.logger;
    }
    
    /**
     * Sub-classes must override this method and does whatever
     * service execution logic.
     * @param inBag input bag
     * @return the service execution result
     * @throws Exception for any exception
     */
    protected abstract Message internalExecute(Message inBag) throws Exception;
    
    /**
     * Sub-classes must override this method for any
     * clear activity.
     * <p>
     * This method will be generally called on container shutdown
     * or application undeplyoment from the container.
     * </p>
     * @param clearInstance any clear instance provied by client
     */
    protected void internalClear(Object clearInstance)
    {
        //Do nothing as default
    }
}
