/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.defne.service.executor;

import java.util.concurrent.atomic.AtomicBoolean;

import javax.naming.InitialContext;

import org.defne.service.IMessageConstants;
import org.defne.service.Message;
import org.defne.service.TransactionPolicy;
import org.defne.utility.factories.SingletonProvider;
import org.defne.utility.log.ILogger;
import org.defne.utility.log.LoggerProvider;

/**
 * This executor executes the given request using the {@link IServiceExecutor}
 * contract. This execution is dones by the service execution bean that is located
 * in the EJB container.
 * 
 * @version $Rev$ $Date$
 *
 */
class EJBServiceExecutorProxy extends AbstractServiceExecutorProxy
{   
    /**Class name of the singleton*/
    public static final String SINGLETON_NAME = "org.defne.service.executor.EJBServiceExecutorProxy";
    
    /**Logger instance*/
    private static final ILogger logger = LoggerProvider.getLogProvider(EJBServiceExecutorProxy.class);
    
    /**EJB service executor local proxy*/
    private IServiceExecutor serviceExecutorProxy = null;
    
    /**This executor is configured or not*/
    private AtomicBoolean proxyConfigured = new AtomicBoolean(false);
        
    /**
     * Creates a new executor.
     */
    EJBServiceExecutorProxy()
    {
        super(logger);
    }
    
    /**
     * Gets singleton executor.
     * @return the singleton executor
     */
    static EJBServiceExecutorProxy getInstance()
    {
        return (EJBServiceExecutorProxy)SingletonProvider.getSingletonInstance(SINGLETON_NAME);
    }
    
    /**
     * {@inheritDoc}
     */
    protected Message internalExecute(Message inputBag) throws Exception
    { 
        Message oBag = null;
        String serviceExecutorJndiName = inputBag.getMessageParameter(String.class, 
                IMessageConstants.SERVICE_EXECUTOR_EJB_JNDI_NAME_KEY);        
        
        if(serviceExecutorJndiName == null)
        {
            throw new NullPointerException("ServiceExecutor EJB JNDI name is required");
        }
        
        if(this.proxyConfigured.compareAndSet(false, true))
        {
            InitialContext context = new InitialContext();
            this.serviceExecutorProxy = (IServiceExecutor) context.lookup(serviceExecutorJndiName);
            
        }

        TransactionPolicy transactionPolicy = inputBag.getMessageParameter(TransactionPolicy.class, 
                IMessageConstants.SERVICE_CALLER_TRANSACTION_POLICY);
        if(transactionPolicy == null)
        {
            transactionPolicy = TransactionPolicy.WITH_TRANSACTION;
            inputBag.putMessageParameter(IMessageConstants.SERVICE_CALLER_TRANSACTION_POLICY, transactionPolicy);
        }
        
        if(transactionPolicy.equals(TransactionPolicy.NEW_TRANSACTION))
        {
            oBag = this.serviceExecutorProxy.executeWithNewTransaction(inputBag);
        }
        else if(transactionPolicy.equals(TransactionPolicy.NO_TRANSACTION))
        {
            oBag = this.serviceExecutorProxy.executeWithoutTransaction(inputBag);
        }
        else
        {
            oBag = this.serviceExecutorProxy.executeWithTransaction(inputBag);
        }
            
        return oBag;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void internalClear(Object clearInstance)
    {
        this.serviceExecutorProxy = null;
    }
    
    
}
